"""Public suggestion engine for incorrect/ambiguous test cases.

This module provides access to the suggestion engine for handling
problematic test cases that require intelligent fixes or improvements.
"""

from __future__ import annotations

from importobot.core.suggestions import GenericSuggestionEngine

__all__ = ["GenericSuggestionEngine"]
